#!/usr/bin/php
<?php

include(dirname(__FILE__).'/config.php');


//Получение аргументов запуска
$getopt_params = array(
    'help'       => '',
	'fast::'     => 'f::',
	'debug::'    => 'd::',
    'value:'     => 'v:',
	'output:'    => 'o:',
	'input:'     => 'i:',
);

$options = getopt( implode('', $getopt_params), array_keys($getopt_params) );

$error_list=array();

if(isset($options['d']) && isset($options['debug'])){
	error_reporting(E_ALL);
}

if(isset($options['help']) || count($options)==0){
	$help='usage: php script.php [--help] [-f|--fast] [-d|--debug] [-o|--output=output_type] [-i|--input=input_type] [-v|--value=checked_value]
About:
        This script converts path to the contents of the site to
        the path to the bared repository and also in the opposite direction

Options:
            --help      Show this message
        -o  --output    Output type. Accepted types:
                           1) Path to bared git repository: "repository"
                           2) Path to content of site: "content"
                           3) Name of this project: "name"
        -i  --input    Input data type. Accepted types:
                           1) Path to bared git repository: "repository"
                           2) Path to content of site: "content"
                           3) Name of this project: "name"
                           4) [default] Auto detected input type: "auto"
        -d  --debug    Enable print php error
        -f  --fast     Disable checking the existence of the output path
        -v  --value

Example:
        php script.php -o repository -v "/var/www/dev/dev.site.local/test/"

Example output:
        /var/www/dev/repositories/test.git/
';

	echo($help);

	OutError('', CODE_ERROR_ARGUMENTS);
}


//Парсинг полученных аргументов
$arguments=array();

if(empty($options['v']) && empty($options['value'])){
	$error_list[]='Argument "value" requred';
}
if(empty($options['o']) && empty($options['output'])){
	$error_list[]='Argument "output" requred';
}

if(count($error_list)==0){
	$arguments['value'] =(!empty($options['v']) ? $options['v'] : $options['value']);
	$arguments['output']=(!empty($options['o']) ? $options['o'] : $options['output']);
	$arguments['input'] =(!empty($options['i']) ? $options['i'] : (!empty($options['input']) ? $options['input'] : 'auto'));
	$arguments['fast']  =(isset($options['f']) || isset($options['fast'])) ? true : false;

	if(!in_array($arguments['output'], array('repository', 'content', 'name'))){
		$error_list[]='Unknown value "'.$arguments['output'].'" of argumnet "output"';
	}

	if(!in_array($arguments['input'], array('auto', 'repository', 'content', 'name'))){
		$error_list[]='Unknown value "'.$arguments['input'].'" of argumnet "input"';
	}
}

if(count($error_list)>0){
	OutError("'".implode("', '", $error_list)."'. See: --help", CODE_ERROR_ARGUMENTS);
}


//Загрузка списка проектов
if(file_exists(CONFIG_PROJECTS_FILE)){
	$projects_lines=file(CONFIG_PROJECTS_FILE);
}elseif(file_exists(dirname(__FILE__).DIRECTORY_SEPARATOR.CONFIG_PROJECTS_FILE)){
	$projects_lines=file(dirname(__FILE__).DIRECTORY_SEPARATOR.CONFIG_PROJECTS_FILE);
}else{
	OutError("Can not load file of projects list", CODE_ERROR_FILE_PROJECTS);
}


//Поиск искомого проекта
$project_found=null;
foreach($projects_lines as $project){
	$project=trim($project);

	if(empty($project)){
		continue;
	}

	if(substr($project, 0, 1)=='#'){
		continue;
	}

	$project_tmp=explode(':', $project, 3);

	if(count($project_tmp)!=3){
		continue;
	}

	$values=array(
		'name'=>$project_tmp[0],
		'repository'=>$project_tmp[1],
		'content'=>$project_tmp[2]
	);

	if($arguments['input']=='auto' || $arguments['input']=='name'){
		if($arguments['value']==$values['name']){
			$project_found=$values;
			break;
		}
	}
	if($arguments['input']=='auto' || $arguments['input']=='repository'){
		if(strpos(MakePath($arguments['value'], 'repository'), MakePath($values['repository'], 'repository'))===0){
			$project_found=$values;
			break;
		}
	}
	if($arguments['input']=='auto' || $arguments['input']=='content'){

		if(strpos(MakePath($arguments['value'], 'content'), MakePath($values['content'], 'content'))===0){
			$project_found=$values;
			break;
		}
	}
}


//Вывод запрошенной информации
if(!is_null($project_found)){
	$result=$project_found[$arguments['output']];

	if($arguments['output']=='repository' || $arguments['output']=='content'){
		$result=MakePath($result, $arguments['output']);

		if(!$arguments['fast']){
			if(!file_exists($result)){
				OutError("Project found, but resulted path does not exist", CODE_ERROR_PATH_NOT_FOUND);
			}
		}
	}

	echo($result);

	exit(0);
}else{
	OutError("Can not find the required project", CODE_ERROR_PROJECT_NOT_FOUND);
}
